/*
 * spi.c
 *
 *  Created on: 21 Aug 2011
 *      Author: Mike
 */

#include <usb.h>
#include <usb_com.h>
#include <radio_queue.h>
#include <stdio.h>
#include <spi.h>

#include <sd.h>

#define CR '\r'
#define LF '\n'
#define null '\0'

#define gVersionString "v1.01 23rd August 2011\n(c) 2011 Mike McPherson"

char getChar(void);
void doCommands(void);
char getChar(void);
int32 getNumber(char *terminator);
void main(void);
void putchar(char c);
void showBuffer( uint8 *buffer, uint16 size);
void showVersion(void);
char skipChars(char toSkip);
void updateLeds(void);

uint32 block;
uint8 verbose;

void doCommands(void)
{
	char charCom;
	int16 i;
	uint8 XDATA buffer[BUFFER_SIZE];
	uint32 startTime, elapsedTime;
	int32 blockRequested;

	charCom = getChar();
	while (charCom != null)
	{
		switch (charCom)
		{
		case '?':
			// identify self
			//==============
			showVersion();
			break;
		case 'd':
			printf("deSelect\n");
			spiDeselect(spiUsart0);
			break;
		case 'i':
			printf("initialise\n");
			startTime = getMs();
			sdInit(spiUsart0);
			elapsedTime = getMs() - startTime;
			block = 0;
			printf("initialise complete in %ld mS\n", elapsedTime);
			break;
		case 'r':
			charCom = ' ';
			blockRequested = getNumber(&charCom);
			if (blockRequested>=0)
				block = blockRequested;
			printf("read block %ld\n", block);
			startTime = getMs();
			i = sdReadBlock(spiUsart0, block, buffer, BUFFER_SIZE);
			if (i != BUFFER_SIZE)
			{
				// write failed :(
				//================
				printf("read block %ld failed: %04X\n", block, i);
			}
			else
			{
				elapsedTime = getMs() - startTime;
				printf("block %ld read in %ld mS\n", block, elapsedTime);
				showBuffer(buffer, BUFFER_SIZE);
				block++;
			}
			break;
		case 's':
			printf("select\n");
			spiSelect(spiUsart0);
			break;
		case 'v':
			// toggle verbose
			//===============
			verbose = !verbose;
			if (verbose)
				printf("verbose on\n");
			else
				printf("verbose off\n");
			break;
		case 'w':
			charCom = ' ';
			i = getNumber(&charCom);
			if (i>=0)
				block = i;
			printf("write block %ld\n", block);
			startTime = getMs();
			spiSelect(spiUsart0);
			i = sdWriteBlock(spiUsart0, block, buffer, BUFFER_SIZE);
			spiDeselect(spiUsart0);
			if (i != BUFFER_SIZE)
			{
				// write failed :(
				//================
				printf("write block %ld failed:%04X\n", block, i);
			}
			else
			{
				elapsedTime = getMs() - startTime;
				printf("block %ld written in %ld mS\n", block, elapsedTime);
				block++;
			}
			break;
		case 'x':
			// generate lots of clocks
			//========================
			printf("clocks\n");
			startTime = getMs();
			while (1)
			{
				spiWriteByte(spiUsart0, 0xFF);
				elapsedTime = getMs() - startTime;
				if (elapsedTime > 10 * 1000)
					break;
			}
			printf("clocks finished\n");
			break;
		case '\r':
			break;
		case '\n':
			break;
		default:
			printf("%c?\n", charCom);
			break;
		}
		if (charCom != CR && charCom != LF)
			printf(">\n");
		charCom = getChar();
	}
}

void doIdle(void)
{
	// service stuff we have to do to keep the world spinning
	//=======================================================
	boardService();
	updateLeds();
	usbComService();
}

char getChar(void)
{
	// function to get the next char from the COM port
	// returns null if nothing in yet
	//================================================

	uint8 charCount;
	uint8 charCom;

	charCount = usbComRxAvailable();

	if (charCount > 0)
	{
		charCom = usbComRxReceiveByte();
		return charCom;
	}
	else
	{
		return(null);
	}
}

int32 getNumber(char *terminator)
{
	char charCom;
	int32 P1;

	// read a number from COM
	// skips leading spaces
	// returns character that terminated the number
	// uses the terminator passed as the first possible character
	// returns -1 by default
	//===========================================================
	P1 = -1;

	// start with character passed
	//============================
	charCom = *terminator;

	// skip leading spaces
	//====================
	if (charCom==' ')
		charCom = skipChars(' ');

	// end of command?
	//================
	if (charCom==CR || charCom==LF)
	{
		// yup, so nothing to do!
		//=======================
		*terminator = charCom;
		return P1;
	}

	// read a P1 as a number
	//======================
	P1 = 0;
	while (charCom!=null && charCom>='0' && charCom<='9')
	{
		P1 = P1 * 10 + charCom - '0';
		charCom = getChar();
	}
	*terminator = charCom;
	return P1;
}

void main(void)
{
	// initialise lots of stuff
	//=========================
	systemInit();
	usbInit();
	radioQueueInit();

	block = 0;

	while (1)
	{
		doIdle();

		// now what we wanted to do!
		//==========================
		doCommands();
	}
}

void putchar(char c)
{
	// This gets called by puts, printf, and printBar
	//===============================================

	// wait for transmit ok
	//=====================
	if (usbComTxAvailable()>0)
	{
		// send a character
		//=================
		usbComTxSendByte(c);
	}
}

void showBuffer(uint8 *buffer, uint16 size)
{
	uint16 i, r, c, x;

	// print the buffer block
	//=======================
	r = 0;
	c = 0;
	do
	{
		if (size-r*16 < 16)
			i = size-r*16;
		else
			i = 16;

		printf("%04X: ", r*16);
		for (c=0; c<i; c++)
		{
			if (c>0)
				printf(".");
			printf("%02X", buffer[r*16+c]);
		}
		printf("  ");
		for (c=0; c<i; c++)
		{
			x = buffer[r*16+c];
			if (x<32 || x>126)
				x = '.';
			printf("%c", x);
		}
		printf("\n");
		r++;
	} while (r*16 < size);
}

void showVersion(void)
{
    printf("\nspi test Interface\n%s\nwixel: %02X-%02X-%02X-%02X\n\n",
    		gVersionString, serialNumber[3], serialNumber[2], serialNumber[1], serialNumber[0]);
}

char skipChars(char toSkip)
{
	char charCom;

	// wait for non-space character from COM port
	// returns 1st non-space character
	//===========================================
	charCom = toSkip;
	while ((charCom == toSkip) || (charCom == null))
	{
		charCom = getChar();
	}
	return charCom;
}

void updateLeds(void)
{
	// housekeeping routine
	//=====================
    usbShowStatusWithGreenLed();
}
